package org.artifactory.build;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mockito;
import org.springframework.mock.web.DelegatingServletInputStream;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;

import static org.artifactory.checksum.ChecksumInfo.TRUSTED_FILE_MARKER;
import static org.artifactory.request.ArtifactoryRequest.CHECKSUM_MD5;
import static org.artifactory.request.ArtifactoryRequest.CHECKSUM_SHA1;
import static org.artifactory.request.ArtifactoryRequest.CHECKSUM_SHA256;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

public class BuildArtifactoryRequestImplTest extends ArtifactoryHomeBoundTest {

    private static final String INITIAL = "initial";
    private static final String REPLACED = "replaced";
    private static final String TEST_HEADER = "TEST-HEADER";

    private HttpServletRequest servletRequest;
    private BuildArtifactoryRequestImpl request;

    @BeforeMethod
    public void setup() throws IOException {
        ArtifactoryContext artifactoryContext = Mockito.mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(artifactoryContext);

        servletRequest = Mockito.mock(HttpServletRequest.class);
        when(servletRequest.getContextPath()).thenReturn("");
        when(servletRequest.getRequestURI()).thenReturn("r");

        InputStream initialStream = new ByteArrayInputStream(INITIAL.getBytes());
        DelegatingServletInputStream delegatingServletInputStream = new DelegatingServletInputStream(initialStream);
        when(servletRequest.getInputStream()).thenReturn(delegatingServletInputStream);

        request = new BuildArtifactoryRequestImpl(servletRequest);
    }

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void whenContentReplacedThenLengthUpdated() throws IOException {
        assertEquals(request.getContentLength(), INITIAL.length());

        request.replaceContent(REPLACED.getBytes());
        assertEquals(request.getContentLength(), REPLACED.length());
    }

    @Test
    public void whenNotChecksumHeaderThenValueFromOriginalRequestReturns() {
        when(servletRequest.getHeaders(TEST_HEADER))
                .thenReturn(Collections.enumeration(Collections.singletonList(TEST_HEADER)));
        assertEquals(request.getHeaderValues(TEST_HEADER).nextElement(), TEST_HEADER);
    }

    @Test
    public void whenChecksumHeaderThenTrustedFileMarkerReturns() {
        assertEquals(request.getHeaderValues(CHECKSUM_SHA1).nextElement(), TRUSTED_FILE_MARKER);
        assertEquals(request.getHeaderValues(CHECKSUM_MD5).nextElement(), TRUSTED_FILE_MARKER);
        assertEquals(request.getHeaderValues(CHECKSUM_SHA256).nextElement(), TRUSTED_FILE_MARKER);
    }

}