package org.artifactory.ph;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.aql.AqlService;
import org.artifactory.aql.api.internal.AqlBase;
import org.artifactory.aql.model.AqlItemTypeEnum;
import org.artifactory.aql.result.AqlEagerResult;
import org.artifactory.aql.result.rows.AqlItem;
import org.artifactory.core.api.ph.model.PackageArtifact;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.openMocks;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;
import static org.testng.Assert.fail;

@Test
public class PackageSearchArtifactServiceImplTest {

    private PackageSearchArtifactServiceImpl packageSearchService;

    @Mock
    private AuthorizationService authorizationService;
    @Mock
    private AqlService aqlService;

    @BeforeMethod
    private void setUp() {
        openMocks(this);
        when(authorizationService.canRead(any())).thenReturn(true);
        this.packageSearchService = new PackageSearchArtifactServiceImpl(aqlService, authorizationService);
    }

    @Test
    public void testFindArtifactsInExactPath() {
        Date date = new Date(System.currentTimeMillis());
        String md5 = UUID.randomUUID().toString().replace("-", "");
        String sha = UUID.randomUUID().toString().replace("-", "");
        AqlEagerResult result = mock(AqlEagerResult.class);
        doAnswer(invocation -> List.of(aqlItem(date, md5, sha + "12345678", sha + sha))).when(result).getResults();
        when(aqlService.executeQueryEager(any(AqlBase.class))).thenReturn(result);

        List<PackageArtifact> arts = packageSearchService.findArtifactsInExactPath("repo", "path", "name")
                .collect(Collectors.toList());
        ArgumentCaptor<AqlBase> aql = ArgumentCaptor.forClass(AqlBase.class);
        verify(aqlService, times(1)).executeQueryEager(aql.capture());
        assertArts(date, md5, sha, arts);
        String query = aql.getValue().toNative(0);
        assertTrue(query.contains("{\"repo\":{\"$eq\":\"repo\"}}"), query);
        assertTrue(query.contains("{\"path\":{\"$eq\":\"path\"}}"), query);
        assertTrue(query.contains("{\"name\":{\"$eq\":\"name\"}}"), query);
        assertTrue(query.contains("{\"depth\":{\"$eq\":2}}"), query);
        assertTrue(query.contains("{\"type\":{\"$eq\":\"file\"}}"), query);
    }

    @Test
    public void testFindChildrenArtifacts() {
        Date date = new Date(System.currentTimeMillis());
        String md5 = UUID.randomUUID().toString().replace("-", "");
        String sha = UUID.randomUUID().toString().replace("-", "");
        AqlEagerResult result = mock(AqlEagerResult.class);
        doAnswer(invocation -> List.of(aqlItem(date, md5, sha + "12345678", sha + sha))).when(result).getResults();
        when(aqlService.executeQueryEager(any(AqlBase.class))).thenReturn(result);

        List<PackageArtifact> arts = packageSearchService.findChildrenArtifacts("repo", "path", "name", 1).collect(
                Collectors.toList());
        ArgumentCaptor<AqlBase> aql = ArgumentCaptor.forClass(AqlBase.class);
        verify(aqlService, times(1)).executeQueryEager(aql.capture());
        assertArts(date, md5, sha, arts);
        String query = aql.getValue().toNative(0);
        assertTrue(query.contains("{\"repo\":{\"$eq\":\"repo\"}}"), query);
        assertTrue(query.contains("{\"path\":{\"$eq\":\"path\"}}"), query);
        assertTrue(query.contains("{\"name\":{\"$eq\":\"name\"}}"), query);
        assertTrue(query.contains("{\"depth\":{\"$gte\":1}}"), query);
        assertTrue(query.contains("{\"type\":{\"$eq\":\"file\"}}"), query);
    }

    @Test
    public void testFindArtifactsByProperties() {
        Date date = new Date(System.currentTimeMillis());
        String md5 = UUID.randomUUID().toString().replace("-", "");
        String sha = UUID.randomUUID().toString().replace("-", "");
        AqlEagerResult result = mock(AqlEagerResult.class);
        doAnswer(invocation -> List.of(aqlItem(date, md5, sha + "12345678", sha + sha))).when(result).getResults();
        when(aqlService.executeQueryEager(any(AqlBase.class))).thenReturn(result);
        Map<String, String> properties = new HashMap<>();
        properties.put("prop.key", "val");
        properties.put("prop.wild", "val*");
        properties.put("prop.emptyVal", "");
        properties.put("prop.nullVal", null);
        List<PackageArtifact> arts =
                packageSearchService.findArtifactsByProperties("repo", "path", properties).collect(
                        Collectors.toList());
        ArgumentCaptor<AqlBase> aql = ArgumentCaptor.forClass(AqlBase.class);
        verify(aqlService, times(1)).executeQueryEager(aql.capture());
        assertArts(date, md5, sha, arts);
        String query = aql.getValue().toNative(0);
        assertTrue(query.contains("{\"repo\":{\"$eq\":\"repo\"}}"), query);
        assertTrue(query.contains("{\"path\":{\"$eq\":\"path\"}}"), query);
        assertTrue(query.contains("{\"@prop.key\":{\"$eq\":\"val\"}}"), query);
        assertTrue(query.contains("{\"@prop.emptyVal\":{\"$eq\":\"\"}}"), query);
        assertTrue(query.contains("{\"@prop.nullVal\":{\"$eq\":null}}"), query);
        assertTrue(query.contains("{\"@prop.wild\":{\"$match\":\"val*\"}}"), query);
        assertTrue(query.contains("{\"type\":{\"$eq\":\"file\"}}"), query);
    }

    @Test
    public void testFindArtifactsByPropertiesInput() {
        Map<String, String> properties = Map.of("prop.key", "v*al");
        try {
            packageSearchService.findArtifactsByProperties("repo", "path", properties);
            fail("Expected IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            assertEquals(e.getMessage(), "Parameter 'v*al' can contain '*' only at the end");
        } catch (Exception e) {
            fail("Expected IllegalArgumentException");
        }
    }

    @Test
    public void testFindArtifactsInExactPathInput() {
        try {
            packageSearchService.findArtifactsInExactPath("repo", "p*ath", "name");
            fail("Expected IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            assertEquals(e.getMessage(), "Parameter 'p*ath' cannot include '*' in exact match search");
        } catch (Exception e) {
            fail("Expected IllegalArgumentException");
        }
    }

    @Test
    public void testFindChildrenArtifactsInput() {
        try {
            packageSearchService.findChildrenArtifacts("repo", "path", "n*am*e", 2);
            fail("Expected IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            assertEquals(e.getMessage(), "Parameter 'n*am*e' cannot contain more than one '*'");
        } catch (Exception e) {
            fail("Expected IllegalArgumentException");
        }
    }

    private AqlItem aqlItem(Date date, String md5, String sha1, String sha2) {
        AqlItem aqlItem = mock(AqlItem.class);
        when(aqlItem.getPath()).thenReturn("path");
        when(aqlItem.getName()).thenReturn("name");
        when(aqlItem.getRepo()).thenReturn("repo");
        when(aqlItem.getSize()).thenReturn(Long.MAX_VALUE);
        when(aqlItem.getSha2()).thenReturn(sha2);
        when(aqlItem.getActualSha1()).thenReturn(sha1);
        when(aqlItem.getNumericCountValue()).thenReturn(Long.MAX_VALUE);
        when(aqlItem.getNodeId()).thenReturn(Long.MAX_VALUE);
        when(aqlItem.getCreated()).thenReturn(date);
        when(aqlItem.getModified()).thenReturn(date);
        when(aqlItem.getUpdated()).thenReturn(date);
        when(aqlItem.getCreatedBy()).thenReturn("admin");
        when(aqlItem.getActualMd5()).thenReturn(md5);
        when(aqlItem.getModifiedBy()).thenReturn("admin");
        when(aqlItem.getOriginalSha1()).thenReturn(sha1);
        when(aqlItem.getType()).thenReturn(AqlItemTypeEnum.file);
        when(aqlItem.getOriginalMd5()).thenReturn(md5);
        when(aqlItem.getDepth()).thenReturn(Integer.MAX_VALUE);
        when(aqlItem.getVirtualRepos()).thenReturn(new String[] {"v1", "v2"});
        return aqlItem;
    }

    private void assertArts(Date date, String md5, String sha, List<PackageArtifact> arts) {
        assertEquals(arts.size(), 1);
        PackageArtifact art = arts.iterator().next();
        assertEquals(art.getRepoKey(), "repo");
        assertEquals(art.getPath(), "path/name");
        assertEquals(art.getCreated(), date.getTime());
        assertEquals(art.getCreatedBy(), "admin");
        assertEquals(art.getLastUpdated(), date.getTime());
        assertEquals(art.getLastModified(), date.getTime());
        assertEquals(art.getId(), Long.MAX_VALUE);
        assertEquals(art.getLength(), Long.MAX_VALUE);
        assertEquals(art.getMd5(), md5);
        assertEquals(art.getModifiedBy(), "admin");
        assertNull(art.getPackageType());
        assertEquals(art.getSha1(), sha + "12345678");
        assertEquals(art.getSha2(), sha + sha);
        assertEquals(art.getName(), "name");
    }
}