package org.artifactory.ph;

import org.artifactory.api.repo.exception.RepoRejectException;
import org.artifactory.checksum.ChecksumType;
import org.artifactory.common.resource.BaseResource;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.fs.RepoResource;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.repo.SaveResourceContext;
import org.artifactory.repo.service.InternalRepositoryService;
import org.artifactory.repo.virtual.VirtualRepo;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.io.IOException;
import java.io.InputStream;

import static org.apache.http.HttpStatus.SC_INTERNAL_SERVER_ERROR;
import static org.apache.http.HttpStatus.SC_NOT_FOUND;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

@Test
public class PackageUploadServiceImplTest {

    private PackageUploadServiceImpl packageUploadService;

    @Mock
    private InternalRepositoryService internalRepositoryService;

    @BeforeMethod
    public void setUp() {
        MockitoAnnotations.openMocks(this);

        packageUploadService = new PackageUploadServiceImpl(internalRepositoryService);
    }

    @Test
    public void testUpload() {
        InputStream is = mock(InputStream.class);
        packageUploadService.upload("repo", "path", is);
        verify(internalRepositoryService, times(1)).deploy(eq(RepoPathFactory.create("repo", "path")), eq(is));
    }

    @Test
    public void testUploadToVirtualCacheRepoNotFound() {
        try {
            InputStream is = mock(InputStream.class);
            packageUploadService.uploadToVirtualCache("repo", "path", is);
            Assert.fail("Should get PackageException:SC_NOT_FOUND");
        } catch (PackageException e) {
            assertEquals(e.getStatus(), SC_NOT_FOUND);
        }
    }

    @Test
    public void testUploadToVirtualCache() throws RepoRejectException, IOException, PackageException {
        VirtualRepo virtualRepo = mock(VirtualRepo.class);
        doAnswer(invocation -> virtualRepo).when(internalRepositoryService).virtualRepositoryByKey(anyString());
        when(internalRepositoryService.saveResourceInTransaction(any(), any())).thenReturn(
                mock(RepoResource.class));
        InputStream is = mock(InputStream.class);
        BaseResource resource = packageUploadService.uploadToVirtualCache("repo", "path", is);
        Assert.assertNotNull(resource);
        ArgumentCaptor<VirtualRepo> vr = ArgumentCaptor.forClass(VirtualRepo.class);
        ArgumentCaptor<SaveResourceContext> rc = ArgumentCaptor.forClass(SaveResourceContext.class);
        verify(internalRepositoryService, times(1)).saveResourceInTransaction(
                vr.capture(), rc.capture());

        assertEquals(vr.getValue(), virtualRepo);
        assertEquals(rc.getValue().getRepoResource().getInfo().getRepoPath(), RepoPathFactory.create("repo", "path"));
        assertEquals(rc.getValue().getRepoResource().getInfo().getChecksumsInfo().size(),
                ChecksumType.BASE_CHECKSUM_TYPES.size());
        assertEquals(rc.getValue().getInputStream(), is);
    }

    @Test
    public void testUploadToVirtualCacheException() throws RepoRejectException, IOException {
        VirtualRepo virtualRepo = mock(VirtualRepo.class);
        doAnswer(invocation -> virtualRepo).when(internalRepositoryService).virtualRepositoryByKey(anyString());
        when(internalRepositoryService.saveResourceInTransaction(any(), any())).thenThrow(new RuntimeException());
        InputStream is = mock(InputStream.class);
        try {
            packageUploadService.uploadToVirtualCache("repo", "path", is);
            Assert.fail("Should get PackageException:SC_INTERNAL_SERVER_ERROR");
        } catch (PackageException e) {
            assertEquals(e.getStatus(), SC_INTERNAL_SERVER_ERROR);
        }
    }
}