package org.artifactory.projects.interceptor;

import com.google.common.collect.ImmutableMap;
import org.artifactory.config.InternalCentralConfigService;
import org.artifactory.descriptor.config.CentralConfigDescriptorImpl;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.HttpRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.VirtualRepoDescriptor;
import org.artifactory.projects.ProjectsService;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jfrog.access.client.AccessClientException;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.anyString;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;
import static org.mockito.Mockito.when;

public class ProjectRepoConfigurationInterceptorTest extends ArtifactoryHomeBoundTest {

    @Mock
    InternalCentralConfigService internalCentralConfigService;
    @Mock
    ProjectsService projectsService;

    @BeforeMethod
    public void setup() {
        MockitoAnnotations.openMocks(this);
    }

    @Test
    public void testOnBeforeSaveWhenNeedToRemoveFromLocalOnly() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("repo", new LocalRepoDescriptor(), "repo2", new LocalRepoDescriptor()));
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        newDescriptor.setLocalRepositoriesMap(ImmutableMap.of("repo", new LocalRepoDescriptor()));
        when(projectsService.getProjectKeyOfRepository("repo2")).thenReturn("prjx");

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService).removeRepositoryFromCurrentProject("repo2");
    }

    @Test
    public void testOnBeforeSaveWhenNeedToRemoveFromFederatedOnly() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setFederatedRepositoriesMap(
                ImmutableMap.of("repo", new FederatedRepoDescriptor(), "repo2", new FederatedRepoDescriptor()));
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        newDescriptor.setFederatedRepositoriesMap(ImmutableMap.of("repo", new FederatedRepoDescriptor()));
        when(projectsService.getProjectKeyOfRepository("repo2")).thenReturn("prjx");

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService).removeRepositoryFromCurrentProject("repo2");
    }

    @Test
    public void testOnBeforeSaveWhenNeedToRemoveFromLocalAndRemote() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("local", new LocalRepoDescriptor(), "deletelocal", new LocalRepoDescriptor()));
        oldDescriptor.setRemoteRepositoriesMap(
                ImmutableMap.of("remote", new HttpRepoDescriptor(), "deleteremote", new HttpRepoDescriptor()));
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        newDescriptor.setLocalRepositoriesMap(ImmutableMap.of("local", new LocalRepoDescriptor()));
        newDescriptor.setRemoteRepositoriesMap(ImmutableMap.of("remote", new HttpRepoDescriptor()));
        when(projectsService.getProjectKeyOfRepository("deletelocal")).thenReturn("prjx");
        when(projectsService.getProjectKeyOfRepository("deleteremote")).thenReturn("prjx");

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService).removeRepositoryFromCurrentProject("deletelocal");
        verify(projectsService).removeRepositoryFromCurrentProject("deleteremote");
        verifyNoMoreInteractions(projectsService);
    }

    @Test
    public void testOnBeforeSaveWhenNeedToRemoveFromLocalAndVirtual() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("local", new LocalRepoDescriptor(), "deletelocal", new LocalRepoDescriptor()));
        oldDescriptor.setVirtualRepositoriesMap(
                ImmutableMap.of("virtual", new VirtualRepoDescriptor(), "deletevirtual", new VirtualRepoDescriptor()));
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        newDescriptor.setLocalRepositoriesMap(ImmutableMap.of("local", new LocalRepoDescriptor()));
        newDescriptor.setVirtualRepositoriesMap(ImmutableMap.of("virtual", new VirtualRepoDescriptor()));
        when(projectsService.getProjectKeyOfRepository("deletelocal")).thenReturn("prjx");
        when(projectsService.getProjectKeyOfRepository("deletevirtual")).thenReturn("prjx");

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService).removeRepositoryFromCurrentProject("deletelocal");
        verify(projectsService).removeRepositoryFromCurrentProject("deletevirtual");
        verifyNoMoreInteractions(projectsService);
    }


    @Test
    public void testOnBeforeSaveWhenNeedToRemoveFromRemoteAndVirtual() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setVirtualRepositoriesMap(
                ImmutableMap.of("virtual", new VirtualRepoDescriptor(), "deletevirtual", new VirtualRepoDescriptor()));
        oldDescriptor.setRemoteRepositoriesMap(
                ImmutableMap.of("remote", new HttpRepoDescriptor(), "deleteremote", new HttpRepoDescriptor()));
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        newDescriptor.setVirtualRepositoriesMap(ImmutableMap.of("virtual", new VirtualRepoDescriptor()));
        newDescriptor.setRemoteRepositoriesMap(ImmutableMap.of("remote", new HttpRepoDescriptor()));
        when(projectsService.getProjectKeyOfRepository("deletevirtual")).thenReturn("prjx");
        when(projectsService.getProjectKeyOfRepository("deleteremote")).thenReturn("prjx");

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService).removeRepositoryFromCurrentProject("deletevirtual");
        verify(projectsService).removeRepositoryFromCurrentProject("deleteremote");
        verifyNoMoreInteractions(projectsService);
    }

    @Test
    public void testOnBeforeSaveWhenThereNoNeedToRemove() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("repo", new LocalRepoDescriptor()));
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        newDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("repo", new LocalRepoDescriptor(), "repo2", new LocalRepoDescriptor()));

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService, never()).getProjectKeyOfRepository(anyString());
        verify(projectsService, never()).removeRepositoryFromCurrentProject(anyString());
    }

    @Test
    public void testOnBeforeSaveWithNullDescriptors() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        when(internalCentralConfigService.getDescriptor()).thenReturn(null);
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        newDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("repo", new LocalRepoDescriptor(), "repo2", new LocalRepoDescriptor()));

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService, never()).getProjectKeyOfRepository(anyString());
        verify(projectsService, never()).removeRepositoryFromCurrentProject(anyString());
    }

    @Test
    public void testOnBeforeSaveWithNullMaps() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setLocalRepositoriesMap(null);
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        newDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("repo", new LocalRepoDescriptor(), "repo2", new LocalRepoDescriptor()));

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService, never()).getProjectKeyOfRepository(anyString());
        verify(projectsService, never()).removeRepositoryFromCurrentProject(anyString());
    }

    @Test
    public void testAccessClientExceptionFromAccess() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("repo", new LocalRepoDescriptor(), "repo2", new LocalRepoDescriptor()));
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        newDescriptor.setLocalRepositoriesMap(ImmutableMap.of("repo", new LocalRepoDescriptor()));
        doThrow(new AccessClientException("Unable to delete")).when(projectsService)
                .removeRepositoryFromCurrentProject(anyString());

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService).removeRepositoryFromCurrentProject("repo2");
    }

    @Test
    public void testNPEFromAccess() {
        ProjectRepoConfigurationInterceptor interceptor = new ProjectRepoConfigurationInterceptor(
                internalCentralConfigService, projectsService);
        CentralConfigDescriptorImpl oldDescriptor = new CentralConfigDescriptorImpl();
        oldDescriptor.setLocalRepositoriesMap(
                ImmutableMap.of("repo", new LocalRepoDescriptor(), "repo2", new LocalRepoDescriptor()));
        CentralConfigDescriptorImpl newDescriptor = new CentralConfigDescriptorImpl();
        when(internalCentralConfigService.getDescriptor()).thenReturn(oldDescriptor);
        newDescriptor.setLocalRepositoriesMap(ImmutableMap.of("repo", new LocalRepoDescriptor()));
        doThrow(new NullPointerException("null")).when(projectsService)
                .removeRepositoryFromCurrentProject(anyString());

        interceptor.onBeforeSave(newDescriptor);

        verify(projectsService).removeRepositoryFromCurrentProject("repo2");
    }
}