package org.artifactory.ui.rest.service.admin.configuration.repositories;

import org.apache.commons.lang.StringUtils;
import org.artifactory.addon.AddonsManager;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.descriptor.config.CentralConfigDescriptorImpl;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.federation.FederatedRepoService;
import org.artifactory.metrics.framework.FeatureAdoptionActions;
import org.artifactory.metrics.framework.FeatureAdoptionFeatures;
import org.artifactory.metrics.framework.FeatureAdoptionService;
import org.artifactory.projects.ProjectsService;
import org.artifactory.repo.RepoDetailsType;
import org.artifactory.rest.common.exception.RepoConfigException;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.rest.exception.ForbiddenException;
import org.artifactory.ui.rest.model.admin.configuration.repository.GeneralRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.CommonLocalRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalRepositoryConfigModel;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.CreateRepoConfigHelper;
import org.artifactory.ui.rest.service.admin.configuration.repositories.util.validator.RepoConfigValidator;
import org.jetbrains.annotations.NotNull;
import org.junit.Assert;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.ArgumentCaptor;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

import static org.junit.Assert.assertEquals;
import static org.junit.jupiter.api.Assertions.fail;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.springframework.test.util.AssertionErrors.assertNotNull;

/**
 * @author AndreiK.
 * @author Shahaf Golan
 */
class CreateRepositoryConfigServiceTest {

    @Mock
    CentralConfigService centralConfigService;
    @Mock
    RepoConfigValidator repoConfigValidator;
    @Mock
    CreateRepoConfigHelper createRepoConfigHelper;
    @Mock
    ProjectsService projectsService;
    @Mock
    FederatedRepoService federatedRepoService;
    @Mock
    FeatureAdoptionService featureAdoptionService;
    @Mock
    AddonsManager addonsManager;

    CreateRepositoryConfigService service;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.openMocks(this);
        ArtifactoryContext context = Mockito.mock(ArtifactoryContext.class);
        when(context.beanForType(FederatedRepoService.class)).thenReturn(federatedRepoService);
        ArtifactoryContextThreadBinder.bind(context);
        service = new CreateRepositoryConfigService(centralConfigService, repoConfigValidator, createRepoConfigHelper,
                projectsService, addonsManager);
    }

    @AfterEach
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @ParameterizedTest
    @MethodSource("repoTypesProvider")
    void testCreateRepo(String modelClass, String repoType) throws RepoConfigException {
        ArtifactoryRestResponse response = new ArtifactoryRestResponse();
        CentralConfigDescriptorImpl descriptor = new CentralConfigDescriptorImpl();
        CommonLocalRepositoryConfigModel model = prepareModel(modelClass, repoType);

        service.createRepo(response, model, descriptor);

        ArgumentCaptor<String> repoTypeCaptor = ArgumentCaptor.forClass(String.class);
        verify(repoConfigValidator).validateRepoName(eq(repoType), repoTypeCaptor.capture());
        assertEquals(repoType, repoTypeCaptor.getValue());
        verify(model).createRepo(createRepoConfigHelper, descriptor);
    }

    @ParameterizedTest
    @MethodSource("repoModelWithNameContainingDashCacheProvider")
    void testCreateRepoFailOnDashCash(String modelClass, String repoKey) throws RepoConfigException {
        ArtifactoryRestResponse response = new ArtifactoryRestResponse();
        CentralConfigDescriptorImpl descriptor = new CentralConfigDescriptorImpl();
        CommonLocalRepositoryConfigModel model = prepareModel(modelClass, repoKey);

        try {
            service.createRepo(response, model, descriptor);
            fail("Expected exception to be thrown");
        } catch (RepoConfigException e) {
            ArgumentCaptor<String> repoTypeCaptor = ArgumentCaptor.forClass(String.class);
            verify(repoConfigValidator).validateRepoName(eq(repoKey), repoTypeCaptor.capture());
            Assert.assertEquals(e.getMessage(),
                    StringUtils.capitalize(repoTypeCaptor.getValue()) + " repository '" + repoKey + "' with " +
                            "'-cache' suffix is not allowed");
            String configModelRepoType = service.getRepoTypeByConfigModel(model);
            assertEquals(configModelRepoType, repoTypeCaptor.getValue());
            verify(model, never()).createRepo(createRepoConfigHelper, descriptor);
        }
    }

    @Test
    void testAddMetricAddingRepoNullRepoType() {
        Assertions.assertDoesNotThrow(() -> {
            service.addMetricRepoCreated(null);
        });
    }

    @Test
    void testAddMetricAddingRepoPubRepoType() {
        service.setFeatureAdoptionService(featureAdoptionService);
        service.addMetricRepoCreated(RepoType.Pub);
        verify(featureAdoptionService).trackAdoption(FeatureAdoptionFeatures.PUB_REPOSITORY,
                FeatureAdoptionActions.CREATE_REPOSITORY_BY_UI_ADMIN, true);
    }

    @Test
    void testFailOnValidateLicense() throws RepoConfigException {
        ArtifactoryRestResponse response = new ArtifactoryRestResponse();
        CentralConfigDescriptorImpl descriptor = new CentralConfigDescriptorImpl();
        CommonLocalRepositoryConfigModel model =
                prepareModel(FederatedRepositoryConfigModel.class.getName(), RepoDetailsType.FEDERATED_REPO);

        String message = "Can't go through with this license";
        doThrow(new ForbiddenException(message)).when(federatedRepoService).validateFederationLicenseInstalled();
        try {
            service.createRepo(response, model, descriptor);
            fail("Expected to get exception");
        } catch (ForbiddenException e) {
            assertEquals(e.getMessage(), message);
        }
    }

    @NotNull
    private CommonLocalRepositoryConfigModel prepareModel(String modelClass, String repoKey) {
        CommonLocalRepositoryConfigModel model = null;
        if (LocalRepositoryConfigModel.class.getName().equals(modelClass)) {
            model = spy(new LocalRepositoryConfigModel());
        } else if (FederatedRepositoryConfigModel.class.getName().equals(modelClass)) {
            model = spy(new FederatedRepositoryConfigModel());
        }
        GeneralRepositoryConfigModel generalModel = new GeneralRepositoryConfigModel();
        generalModel.setRepoKey(repoKey);
        assertNotNull("Unrecognized class name was provided", model);
        model.setGeneral(generalModel);
        return model;
    }

    static Object[][] repoTypesProvider() {
        return new Object[][] {
                {LocalRepositoryConfigModel.class.getName(), RepoDetailsType.LOCAL_REPO},
                {FederatedRepositoryConfigModel.class.getName(), RepoDetailsType.FEDERATED_REPO},
        };
    }

    static Object[][] repoModelWithNameContainingDashCacheProvider() {
        return new Object[][] {
                {LocalRepositoryConfigModel.class.getName(), RepoDetailsType.LOCAL_REPO + "-cache"},
                {FederatedRepositoryConfigModel.class.getName(), RepoDetailsType.FEDERATED_REPO + "-cache"},
        };
    }
}