package org.artifactory.rest.resource.system;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.HaAddon;
import org.artifactory.aol.AolService;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.rest.exception.BadRequestException;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jfrog.common.YamlUtils;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

public class ConfigResourceTest extends ArtifactoryHomeBoundTest {
    @Mock
    private CentralConfigService centralConfigService;
    @Mock
    private HttpServletRequest request;
    @Mock
    private CentralConfigDescriptor descriptor;
    @Mock
    private ArtifactoryContext context;
    @Mock
    private AolService aolService;
    @Mock
    AddonsManager addonsManager;
    @Mock
    HttpServletRequest servletRequest;

    private AutoCloseable autoCloseable;
    private ConfigResource configResource;

    @BeforeMethod
    private void setup() {
        this.autoCloseable = MockitoAnnotations.openMocks(this);
        when(centralConfigService.getDescriptor()).thenReturn(descriptor);
        this.configResource = new ConfigResource(centralConfigService, servletRequest, aolService, addonsManager);
    }

    @AfterMethod
    private void cleanup() throws Exception {
        this.autoCloseable.close();
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testIsSaas() {
        this.createAolMock(true, false);
        assertTrue(this.configResource.isSaas().isSaas());
    }

    @Test
    public void testIsNotSaas() {
        this.createAolMock(false, false);
        assertFalse(this.configResource.isSaas().isSaas());
    }

    @Test(expectedExceptions = BadRequestException.class)
    public void testGetResourceAolNotDashboardUser() {
        this.createAolMock(true, false);
        this.configResource.getConfig();
    }

    @Test(dataProvider = "configure")
    public void testGetResourceAolDashboardUser(boolean isAol, boolean isDashboardUser) {
        this.createAolMock(isAol, isDashboardUser);
        Assert.assertNotNull(this.configResource.getConfig());
    }

    @DataProvider
    public Object[][] configure() {
        return new Object[][] {
                {true, true},
                {false, false}
        };
    }

    private void createAolMock(boolean isAol, boolean isDashboardUser) {
        when(this.aolService.isDashboardUser()).thenReturn(isDashboardUser);
        when(this.aolService.isAol()).thenReturn(isAol);
        when(this.context.beanForType(AolService.class)).thenReturn(this.aolService);
        ArtifactoryContextThreadBinder.bind(this.context);
    }

    @Test
    public void testSetConfigAolDashboardUser() {
        when(aolService.isAol()).thenReturn(true);
        when(aolService.isDashboardUser()).thenReturn(true);
        ConfigResource configResource =
                new ConfigResource(centralConfigService, servletRequest, aolService, addonsManager);
        configResource.setConfig("Some config");
    }

    @Test(expectedExceptions = BadRequestException.class)
    public void testSetConfigAolNotDashboardUser() {
        when(aolService.isAol()).thenReturn(true);
        when(aolService.isDashboardUser()).thenReturn(false);
        HaAddon haAddon = mock(HaAddon.class);
        when(haAddon.isHaAuthentication()).thenReturn(false);
        when(addonsManager.addonByType(HaAddon.class)).thenReturn(haAddon);
        ConfigResource configResource =
                new ConfigResource(centralConfigService, servletRequest, aolService, addonsManager);
        configResource.setConfig("Some config");
    }

    @Test
    public void testSetConfigAolHaUser() {
        when(aolService.isAol()).thenReturn(true);
        when(aolService.isDashboardUser()).thenReturn(false);
        HaAddon haAddon = mock(HaAddon.class);
        when(haAddon.isHaAuthentication()).thenReturn(true);
        when(addonsManager.addonByType(HaAddon.class)).thenReturn(haAddon);
        ConfigResource configResource =
                new ConfigResource(centralConfigService, servletRequest, aolService, addonsManager);
        configResource.setConfig("Some config");
    }

    @Test
    public void testPatchConfigAolDashboardUser() {
        when(aolService.isAol()).thenReturn(true);
        when(aolService.isDashboardUser()).thenReturn(true);
        ConfigResource configResource =
                new ConfigResource(centralConfigService, servletRequest, aolService, addonsManager);
        Map<Object, Object> data = ImmutableMap.builder()
                .put("indexer",
                        ImmutableMap.of(
                                "enabled", "false",
                                "cronExp", "0 0 0 0 0",
                                "includedRepositories", ImmutableList.of("repokey20", "repokey21"))
                )
                .build();
        configResource.patchConfig(YamlUtils.getInstance().writeValueAsString(data));
    }

    @Test
    public void testPatchConfigAolHaUser() {
        when(aolService.isAol()).thenReturn(true);
        when(aolService.isDashboardUser()).thenReturn(false);
        HaAddon haAddon = mock(HaAddon.class);
        when(haAddon.isHaAuthentication()).thenReturn(true);
        when(addonsManager.addonByType(HaAddon.class)).thenReturn(haAddon);
        ConfigResource configResource =
                new ConfigResource(centralConfigService, servletRequest, aolService, addonsManager);
        Map<Object, Object> data = ImmutableMap.builder()
                .put("indexer",
                        ImmutableMap.of(
                                "enabled", "false",
                                "cronExp", "0 0 0 0 0",
                                "includedRepositories", ImmutableList.of("repokey20", "repokey21"))
                )
                .build();
        configResource.patchConfig(YamlUtils.getInstance().writeValueAsString(data));
    }
}